//last Update 06/09/2017
//03-09-2017  reduction des delais
// Fosc 16MHz sur MCU

#define LCD_ADDR 0x4E     // PCF8574 on pigboard YwRobot back located  on LCD4x20
// PCF8574 I2C LCD Port Definition
// P7,P6,P5,P4 = Data, P3=Backlight (BL), P2=E, P1=RW, P0=RS
#define LCD_BL 0x08
#define LCD_EN 0x04
#define LCD_RW 0x02
#define LCD_RS 0x01

// ref : ODT OCM-20416D-A2023 LCD Module  8/4/1999
// LCD Command
#define LCD_CLEAR 0x01
#define LCD_HOME  0x02
#define Display_ON 0x0C
#define Display_OFF 0x08
#define LCD_LINE1 0x80      // 0x00 + bit D7=1
#define LCD_LINE2 0xC0      // 0x40 + bit D7=1
#define LCD_LINE3 0x94      // 0x14 + bit D7=1
#define LCD_LINE4 0xD4      // 0x54 + bit D7=1

extern void strConstRamCpy(unsigned char *dest, const code char *source);
extern void CRLF(void) ;

unsigned char I2C_PCF8574_Write(unsigned char  Adr,unsigned char value);
void LCD_init_4x20(void);
void LCD_putch(unsigned char d); 
void LCD_Chr(unsigned char L,unsigned char x,unsigned char d);  
void Go_To_Line_Col( unsigned char L, unsigned char Col);
void LCD_puts(char *s);
void LCD_putsRom(code char *s);
void Lcd_Out(unsigned char L,unsigned char x,const code char *CT);
void LCD_Cmd(unsigned char d);
void LCD_One_Cmd(unsigned char d);   // uniquement pour Init LCD

//fonctions LCD derives
void LCD_Msg_Status(const char* T);  // affiche un TEXTE en ligne 4
void LCD_Write_CText(const char *txt) ;
void LCDHandler(char c);


//======= LCD 4x20 sur bus I2C ======
 /*
  PCF8574 I2C LCD Routine
  LCD Data PCA8574: P7,P6,P5,P4
  LCD Control: P3: Back Light, P2: E-Enable, P1:RW, P0: RS
*/

 unsigned char I2C_PCF8574_Write(unsigned char Adr,unsigned char value)
{
 unsigned char S,dummy;
 I2C1_Start();
 S = I2C1_Wr( Adr );
 if(S == -1)        //check if bus collision happened
        {
       dummy = SSPBUF;        //upon bus collision detection clear the buffer,
       SSPCON1.WCOL=0;    // clear the bus collision status bit
        }
S = I2C1_Wr(value);
I2C1_Stop();
return(S);
}


void Erase_Line(int L)   // new
{
switch(L)
 {
 case 1:
  {
   LCD_Cmd(LCD_LINE1);
   LCD_Write_CText(mesg0);
   LCD_Cmd(LCD_LINE1);
   break;
  }
  case 2:
  {
   LCD_Cmd(LCD_LINE2);
   LCD_Write_CText(mesg0);
   LCD_Cmd(LCD_LINE2);
    break;
  }
  case 3:
  {
   LCD_Cmd(LCD_LINE3);
   LCD_Write_CText(mesg0);
   LCD_Cmd(LCD_LINE3);
    break;
  }
  case 4 :
  {
   LCD_Cmd(LCD_LINE4);
   LCD_Write_CText(mesg0);
   LCD_Cmd(LCD_LINE4);
    break;
  }
 }
}

void Go_To_Line_Col( unsigned char L, unsigned char Col)
{
 switch(L)
 {
 case 1:
  {
   LCD_Cmd(LCD_LINE1+Col-1);
   break;
  }
  case 2:
  {
   LCD_Cmd(LCD_LINE2+Col-1);
    break;
    }
  case 3:
  {
   LCD_Cmd(LCD_LINE3+Col-1);
    break;
  }
  case 4 :
  {
   LCD_Cmd(LCD_LINE4+Col-1);
    break;
  }
 }
 }




void LCD_Out(unsigned char L,unsigned char x,const code char *CT)
 {
 unsigned char lcddata;
 Go_To_Line_Col( L,x);
  Delay_ms(15);
  LCD_Write_CText(CT);
 }


 void LCD_One_Cmd(unsigned char d)
{
  unsigned char lcddata;
  // Put the Upper 4 bits data
  lcddata = (d & 0xF0)|LCD_BL;
  Etat=I2C_PCF8574_Write(Adr,lcddata | LCD_EN);
  Delay_us(5);
  // Write Enable Pulse E: Hi -> Lo
  Etat=I2C_PCF8574_Write(Adr,lcddata & ~LCD_EN);
  Delay_us(5);      // Delay 1us for 16 MHz Internal Clock
}


void LCD_Cmd(unsigned char d)
{
  unsigned char lcddata;
  lcddata = (d & 0xF0)|LCD_BL;
  Etat=I2C_PCF8574_Write(Adr,lcddata | LCD_EN);
  Delay_us(5);
  Etat=I2C_PCF8574_Write(Adr,lcddata & ~LCD_EN);
  Delay_us(5);      // Delay 1us for 16 MHz Internal Clock
    lcddata = ((d << 4) & 0xF0)|LCD_BL;
    Etat=I2C_PCF8574_Write(Adr,lcddata | LCD_EN);
    Delay_us(2);    // Delay 2us for 16 MHz Internal Clock
    Etat=I2C_PCF8574_Write(Adr,lcddata & ~LCD_EN);
    Delay_ms(2);    // Delay 1us for 16 MHz Internal Clock

}



void LCD_putch(unsigned char d)
{
  unsigned char lcddata;
  // Put the Upper 4 bits data
  lcddata = (d & 0xF0)|LCD_BL|LCD_RS;
  Etat=I2C_PCF8574_Write(Adr,lcddata | LCD_EN);
  Delay_us(2);      // Delay 2us for 16 MHz Internal Clock
  // Write Enable Pulse E: Hi -> Lo
  Etat=I2C_PCF8574_Write(Adr,lcddata & ~LCD_EN);
  Delay_us(2);      // Delay 1us for 16 MHz Internal Clock
  // Put the Lower 4 bit data
  lcddata = ((d << 4) & 0xF0)|LCD_BL|LCD_RS;
  Etat=I2C_PCF8574_Write(Adr,lcddata | LCD_EN);
  Delay_us(2);      // Delay 2us for 16 MHz Internal Clock
  // Write Enable Pulse E: Hi -> Lo
  Etat=I2C_PCF8574_Write(Adr,lcddata & ~LCD_EN);
  Delay_us(2);      // Delay 1us for 16 MHz Internal Clock
}

void LCD_Chr(unsigned char L,unsigned char x,unsigned char d)
{
  unsigned char lcddata;
   Go_To_Line_Col( L,x);
  Delay_ms(10);
   LCD_putch(d);
  
}

void LCD_init_4x20(void)
{
 Adr=LCD_ADDR;
  // Wait for more than 15 ms after VCC rises to 4.5 V
  Delay_ms(500);
  // Send Command 0x30
  LCD_One_Cmd(0x30);
  // Wait for more than 4.1 ms
  Delay_ms(5);
  // Send Command 0x30
  LCD_One_Cmd(0x30);
  // Wait for more than 100 us
  Delay_ms(2);
  // Send Command 0x30
  LCD_One_Cmd(0x30);
    Delay_ms(2);
  // Function set: Set interface to be 4 bits long (only 1 cycle write).
  LCD_One_Cmd(0x20);
    Delay_ms(2);
  // Function set: DL=0;Interface is 4 bits, N=1; 2 Lines, F=0; 5x8 dots font)
  LCD_Cmd(0x28);
    Delay_ms(2);
  // Display Off: D=0; Display off, C=0; Cursor Off, B=0; Blinking Off
  LCD_Cmd(0x08);
    Delay_ms(2);
  // Display Clear
  LCD_Cmd(LCD_CLEAR);
    Delay_ms(100);
  // Entry Mode Set: I/D=1; Increament, S=0; No shift
  LCD_Cmd(0x06);
    Delay_ms(20);
  // Display On, Cursor Off
  LCD_Cmd(0x0C);
    Delay_ms(20);
}

void LCD_putsRom(const code char *s)
{int i=0;
  while((*s != 0)&& (i<20))
  {
    LCD_putch(*s);
    s++;i++;
  }
}

void LCD_puts( char *s)
{int i=0;
  while((*s != 0)&& (i<20))
  {
    LCD_putch(*s);
    s++;   i++;
  }
}

void LCD_Write_CText(const char *txt)
{
   while (*txt)
   LCD_putch(*txt++);
 }

void LCDHandler(char c) {
  LCD_putch(c);
}

 void LCD_Msg_Status(const char* T)
{
// utilise la table TEXE[] !
 LCD_Cmd(LCD_LINE4);
// LCD_putcmd(LCD_LINE4,1);
 strConstRamCpy(txt,T);
 //UART1_Write_Text(txt);     CRLF();
 LCD_puts(txt);
}
